"""
        GNU AFFERO GENERAL PUBLIC LICENSE
           Version 3, 19 November 2007

 Copyright (C) 2007 Free Software Foundation, Inc. <https://fsf.org/>
 Everyone is permitted to copy and distribute verbatim copies
 of this license document, but changing it is not allowed.

 THIS IS A PART OF MONEY ON CHAIN
 @2020
 by Martin Mulone (martin.mulone@moneyonchain.com)

"""

import os
import logging
from web3.types import BlockIdentifier
from web3 import Web3

from moneyonchain.contract import ContractBase


class ProxyAdmin(ContractBase):

    contract_name = 'ProxyAdmin'
    contract_abi = ContractBase.content_abi_file(
        os.path.join(os.path.dirname(os.path.realpath(__file__)), 'abi/ProxyAdmin.abi'))
    contract_bin = ContractBase.content_bin_file(
        os.path.join(os.path.dirname(os.path.realpath(__file__)), 'abi/ProxyAdmin.bin'))

    mode = 'MoC'
    precision = 10 ** 18

    def __init__(self,
                 network_manager,
                 contract_name=None,
                 contract_address=None,
                 contract_abi=None,
                 contract_bin=None):

        if not contract_address:
            config_network = network_manager.config_network
            contract_address = network_manager.options['networks'][config_network]['addresses']['admin']

        super().__init__(network_manager,
                         contract_name=contract_name,
                         contract_address=contract_address,
                         contract_abi=contract_abi,
                         contract_bin=contract_bin)

    def implementation(self,
                       contract_address,
                       block_identifier: BlockIdentifier = 'latest'):
        """Get proxy implementation"""

        result = self.sc.getProxyImplementation(contract_address, block_identifier=block_identifier)

        return result


def admin_implementation(network_manager, contract_address, block_identifier: BlockIdentifier = 'latest'):
    """Implementation of contract"""

    contract_admin = ProxyAdmin(network_manager).from_abi()
    contract_address = Web3.toChecksumAddress(contract_address)

    return contract_admin.implementation(contract_address, block_identifier=block_identifier)


class ProxyAdminInterface(ContractBase):
    def implementation(self, block_identifier: BlockIdentifier = 'latest'):
        """Implementation of contract"""

        return admin_implementation(self.network_manager,
                                    self.contract_address,
                                    block_identifier=block_identifier)
